@@ -35,8 +35,8 @@
 #define BRANDY_NAME  "Matrix"
 #define BRANDY_MAJOR "1"
 #define BRANDY_MINOR "23"
-#define BRANDY_PATCHLEVEL "2"
-#define BRANDY_DATE       "22 May 2024"
+#define BRANDY_PATCHLEVEL "3"
+#define BRANDY_DATE       "06 Aug 2024"
 // #define BRANDY_PATCHDATE  "JGH191007"
 
 /*
@@ -207,6 +207,8 @@ typedef unsigned long int nativeuint;   /* 32 or 64-bit depending on architectur
 #define DEFAULT_EDITOR  "vi"
 #define DIR_SEPS "/"
 #define DIR_SEP  '/'
+#define STRLCPY(x,y,z) strlcpy(x,y,z)
+#define STRLCAT(x,y,z) strlcat(x,y,z)
 #endif
 
 #ifdef linux
@@ -446,6 +448,15 @@ typedef unsigned long int nativeuint;   /* 32 or 64-bit depending on architectur
 typedef jmp_buf sigjmp_buf;
 #endif /* TARGET_MINGW || __TARGET_SCL__ */
 
+/* OpenBSD recommends strlcpy() and frends over strncpy(), but it's not
+ * available on all systems. So macro it to resolve as strlcpy on OpenBSD
+ * and strncpy on others. Also, strlcat() takes into account the current
+ * destination string length, strncat() does not. */
+#ifndef STRLCPY
+#define STRLCPY(x,y,z) my_strlcpy(x,y,z)
+#define STRLCAT(x,y,z) strncat(x,y,z - strlen(x) -1)
+#endif /* STRLCPY */
+
 #ifdef __TARGET_SCL__
 /* FIXME: Implement this properly! */
 #define usleep(x)
